// Copyright 1996-1999 Omni Development, Inc.  All rights reserved.
//
// This software may only be used and reproduced according to the
// terms in the file OmniSourceLicense.html, which should be
// distributed with this project and can also be found at
// http://www.omnigroup.com/DeveloperResources/OmniSourceLicense.html.


#define RCS_ID(rcsIdString) \
       static void *rcs_id = rcsIdString; \
       static void *__rcs_id_hack() { __rcs_id_hack(); return rcs_id; }

RCS_ID("$Header: /Network/Source/CVS/OmniGroup/Bundles/InterfaceBuilder/OmniCVSPalette/CVSPalette.m,v 1.4 1999/10/11 21:32:04 andrew Exp $")


#import <Foundation/Foundation.h>
#import <AppKit/AppKit.h>
#import <InterfaceBuilder/IBDocuments.h>

#import "CVSPalette.h"


@interface NSFileManager (CVSPaletteExtensions)
- (BOOL)directoryExistsAtPath:(NSString *) path;
@end


@implementation NSFileManager (CVSPaletteExtensions)

- (BOOL)directoryExistsAtPath:(NSString *)path;
{
    BOOL isDirectory;

    if (![self fileExistsAtPath:path isDirectory:&isDirectory])
        return NO;
    return isDirectory;
}

@end


@interface CVSPalette (Private)
- (NSString *)_didSaveDocument:(NSNotification *)notification;
@end


@implementation CVSPalette

- init
{
    [super init];
    
   /*
    * Register ourselves as an IBDocumentController so we will receive
    * didSaveDocument: messages from IB whenever the user saves a document.
    *
    * The registration had to be moved here from finishInstantiate
    * because finishInstantiate does not get called until you click
    * on the palette icon. This could result in the palette never
    * getting registered and, therefore, the CVS admin files would not
    * get moved.
    */
    NSLog(@"Registering %@ as a documentController", self);
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(didSaveDocument:)
                                                 name:IBDidSaveDocumentNotification
                                               object:nil];
	 
    return self;
}

// Here's the method that actually gets the notification.
- (void)didSaveDocument:(NSNotification *)notification;
{
    NSString *error;
    
    if ((error = [self _didSaveDocument:notification]))
        NSRunAlertPanel(@"OmniCVSPalette Error", error, nil, nil, nil);
}

@end


@implementation CVSPalette (Private)

// Here's the method that does the work.
- (NSString *)_didSaveDocument:(NSNotification *)notification;
{
    NSString *path;
    id <IBDocuments> document;
    NSFileManager *manager;
    NSString *cvsPath;
    NSString *oldPath;

    document = (id <IBDocuments>) [notification object];
    path = [document documentPath];
    manager = [NSFileManager defaultManager];

    // make sure we can find the destination (new) nib (if not, really strange stuff going on)
    if (![manager directoryExistsAtPath:path])
        return [NSString stringWithFormat:@"Could not find nib file at %@", path];

    // try to find the backup (old) nib
    oldPath = [path stringByAppendingString:@"~"];
    if (![manager directoryExistsAtPath:oldPath])
        return nil;	// presumably a brand new nib - nothing to do

    // try to find the CVS directory in the backup (old) nib
    cvsPath = [oldPath stringByAppendingPathComponent:@"CVS"];
    if (![manager directoryExistsAtPath:cvsPath])
        return nil;	// no CVS info to move - probably never been checked in

    // make sure there isn't already a CVS directory in the destination (new) nib
    // (if so, something strange going on - some other CVS palette already took charge, maybe?)
    path = [path stringByAppendingPathComponent:@"CVS"];
    if ([manager directoryExistsAtPath:path])
        return [NSString stringWithFormat:@"CVS directory already exists at %@", path];

    // move path rather than copy it so the old nib doesn't look like something that CVS cares about
    if (![manager movePath:cvsPath toPath:path handler:nil])
        return [NSString stringWithFormat:@"Failed to move '%@' to '%@'", cvsPath, path];

    return nil;
}

@end
